<?php

/**
* @copyright	Copyright (C) 2015 Codextrous . All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package 		JOOMDOWNLOADS
* @subpackage	Back-end
* @contact		contact@codextrous.com
*/

// no direct access
defined('_JEXEC') or die;

/** 
 * Release Controller
 * @author Gaurav Jain
 */
class JOOMDownloadsAdminControllerRelease extends JOOMDownloadsController 
{
	public function _save(array $data, $itemId=null)
	{

		$model = $this->getModel();
		$data['version'] = self::_sanitize_version($data['version']);
		if (trim(str_replace('-', '', $data['version'])) == '')
		{
			$this->setError(JText::_('COM_JOOMDOWNLOADS_INVALID_VERSION'));
			return false;
		}

		$records = $model->loadRecords(array('product_id' => $data['product_id'], 'version' => $data['version']));
		if(!empty($records)){
			if(!empty($itemId)){
				foreach($records as $record){
					if($itemId != $record->release_id){
						$this->setError(JText::_('COM_JOOMDOWNLOADS_VERSION_ALREADY_EXIST_FOR_PRODUCT'));
						return false;
					}
				}
			}
			else{
				$this->setError(JText::_('COM_JOOMDOWNLOADS_VERSION_ALREADY_EXIST_FOR_PRODUCT'));
				return false;
			}
		}

		$release  = parent::_save($data, $itemId);

        if($release){
            $releasedata = $release->toArray();
            $id 	 = $releasedata['release_id'];

            //delete and save access records
            $model = $this->getModel();
            $model->deleteAccessRecords($id);
            if(isset($data['access']) && !empty($data['access'])){
                $model->insertAccessRecords($id, $data['access']);
            }
        }

		if($release && !empty($_FILES['joomdownloads_form']['tmp_name']['uploadfile'])){
			if(!JFolder::exists(JOOMDOWNLOADS_PATH_RELEASE)){
				JFolder::create(JOOMDOWNLOADS_PATH_RELEASE);
			}
			
			$file = array();
			foreach($_FILES['joomdownloads_form'] as $key => $value){
				$file[$key] = $value['uploadfile'];
			}

			if (isset($file['name'])) {
				$file['name']	= JFile::makeSafe($file['name']);
			}

			$utils	= JOOMDownloadsFactory::getHelper('utils');
			$error = '';
			
			$url = JRoute::_('index.php?option=com_joomdownloads&view=release&task=edit&id='.$release->getId(), false);
			$app = JFactory::getApplication();
			$canUpload = $utils->canUpload($file, $error);
			if($canUpload){
				$storage_path = JOOMDOWNLOADS_PATH_RELEASE.'/';
				
				// check if file already exists
				if(JFile::exists($storage_path.$file['name'])){
					$app->redirect($url, JText::_('COM_JOOMDOWNLOADS_FILE_ALREADY_EXISTS'), 'error');
					return false;
				}

				$filename = $utils->saveUploadedFile($storage_path,$file['tmp_name'],$file['name']);
				if($filename === false){
					$app->redirect($url, JText::_('COM_JOOMDOWNLOADS_FILE_ERROR_IN_COPYING'), 'error');
					return false;
				}
				$release->bind(array('filename' => $filename))->save();
			}
			else{
				if(!empty($error)){
					$app->redirect($url, JText::_($error), 'error');
					return false;
				}
				else{
					$app->redirect($url, JText::_('COM_JOOMDOWNLOADS_ADMIN_UNKNOWN_ERROR'), 'error');
					return false;	
				}
			}

		}

		return $release;
	}

	protected function _sanitize_version( $string, $force_lowercase = true, $anal = false) {
	    $strip = array("~", "`", "!", "@", "#", "$", "%", "^", "&", "*", "(", ")", "_", "=", "+", "[", "{", "]",
	                   "}", "\\", "|", ";", ":", "\"", "'", "&#8216;", "&#8217;", "&#8220;", "&#8221;", "&#8211;", "&#8212;",
	                   "Ã¢â‚¬â€?", "Ã¢â‚¬â€œ", ",", "<", ">", "/", "?");
	    $clean = trim(str_replace($strip, "", strip_tags($string)));
	    $clean = preg_replace('/\s+/', "-", $clean);
	    $clean = ($anal) ? preg_replace("/[^a-zA-Z0-9]/", "", $clean) : $clean ;
	    return ($force_lowercase) ?
	        (function_exists('mb_strtolower')) ?
	            mb_strtolower($clean, 'UTF-8') :
	            strtolower($clean) :
	        $clean;
	}
}