/**
* @copyright	Copyright (C) 2015 Codextrous . All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		JD Framework
* @subpackage	UI
* @contact 		dev.codextrous@gmail.com
*/

/*-----------------------------------------------------------
  Javascript writing standards - 
  - Pack you code in 
  		(function($){
  				// Your code here
  				// Your code here
  		})(jd.jQuery);
  		
  	and use $ as usually.
-----------------------------------------------------------*/
if (typeof(jd)=='undefined')
{
	var jd = {
		jQuery: window.jQuery,
		extend: function(obj){
			this.jQuery.extend(this, obj);
		}
	}
}


(function($){
// START : 	
// Scoping code for easy and non-conflicting access to $.
// Should be first line, write code below this line.	
	

/*--------------------------------------------------------------
  UI related works
  jd.ui.dialog.create  = create a dialog, fill with ajax data
  jd.ui.dialog.button  = add buttons on dialog
  jd.ui.dialog.title   = set title
  jd.ui.dialog.height  = set height
  jd.ui.dialog.close   = close dialog
--------------------------------------------------------------*/
jd.ui = {};
jd.ui.dialog = {
	create : function(call, winTitle, winContentWidth, winContentHeight, winContentLeft){

		if(winTitle == null) winTitle = 'Title';
		if(winContentWidth == null) winContentWidth = 'auto';
		if(winContentHeight == null) winContentHeight = 'auto';
		
		// create a empty-div & show a dialog
		$('#jdWindowContent').remove();

		//XITODO : loading class required or not
		$('<div id="jdWindowContent" class="modal hide fade loading" tabindex="-1" role="dialog" aria-labelledby="myModalLabel" aria-hidden="true"></div>')
				.addClass('new').appendTo('.jd-wrap');
		
		// add title, body and footer block so that sequence can be maintained
		$('<div id="jdWindowTitle"></div>').appendTo('#jdWindowContent');
		$('<div id="jdWindowBody"></div>').appendTo('#jdWindowContent');
		$('<div id="jdWindowFooter"></div>').appendTo('#jdWindowContent');
		
		// set the title
		this.title(winTitle);
		$('#jdWindowContent').on('show', function () {
			var modal = $(this);
			
			//@jdTODO:: Should be responsive
			// Customize height and width.			
			// Centralize modal window after set required width and height.			
			modal.css({	width:winContentWidth, height:winContentHeight, left:winContentLeft });
				 //.css('margin-top', (modal.outerHeight() / 2) * -1)				 
		         //.css('margin-left', (modal.outerWidth() / 2) * -1);
			
		    return this;
		    
		});
		
		// on hiding the popup, remove the div#jdWindowContent also
		$('#jdWindowContent').on('hidden', function() {
			$('#jdWindowContent').remove();
		});
		
		// show the modal
		$('#jdWindowContent').modal('show');
		
		// call ajax
		if(call != null){
			jd.ajax.go(call.url, call.data);
		}
		return true;
	},
	
	button : function(actions){
		// empty previous action buttons
		$('#jdWindowFooter').html('');
		$('<div class="modal-footer"></div>').appendTo('#jdWindowFooter');

		for(var i=0;i<actions.length;i++) {
			actions[i].click = eval("(function(){" + actions[i].click + ";})" );
			var button = '<button class="'+actions[i].classes+'" '+actions[i].attr+'>'+actions[i].text+'</button>';
			$(button).bind('click', actions[i].click).appendTo('#jdWindowFooter > .modal-footer');
		}		
	},
	
	body : function(body){
		// empty previous body content
		$('#jdWindowBody').html('');
		if(body != null && body.length > 0){
			$('<div class="modal-body"><p>'+body+'</p></div>').appendTo('#jdWindowBody');
		}
	},
	
	title : function(title){
		// empty previous title
		$('#jdWindowTitle').html('');
		
		// show the header in case of title is not empty
		if(title != null && title.length > 0){
			$('<div class="modal-header"><button type="button" class="close" data-dismiss="modal" aria-hidden="true">×</button><h3 id="myModalLabel">'+title+'</h3></div>')
				.appendTo('#jdWindowTitle');
		}
	},
	
	close : function(title){
		$('#jdWindowContent').modal('hide');
	},
	
	height : function(height){
		$('#jdWindowBody').css("height", height);
	},

	width : function(width){
		$('#jdWindowContent').css("width", width);
	},

	autoclose : function($time){
		setTimeout(function(){
			$('#jdWindowContent').modal('hide')
		}, $time);
	}
};

/*--------------------------------------------------------------
  URL related work
  	# url.modal		:	Open a url in modal window
   	# url.redirect	:	Redirect current window to new url
--------------------------------------------------------------*/
//deprecated and move into jd.url
jd.route = {
	url : function(url){
				return jd.url.route(url)
	}
};


jd.url = {
	  	modal	: 	function( theurl, options, windowWidth, windowHeight)
	  	{
			var ajaxCall = {'url':theurl, 'data': {}, 'iframe': false};
				
			if(options) {
			  ajaxCall = {'url':theurl, 'data':options.data, 'iframe' : false};
			}
			
			if (!windowWidth)  { windowWidth = 'auto';	 }
			
			if (!windowHeight) { windowHeight = 'auto'; }

			jd.ui.dialog.create(ajaxCall, '', windowWidth, windowHeight);
		},
			
		redirect	:	function(url) 
		{
			document.location.href=url;
		},
		
		route	:	function(url)
		{
			// already a complete URL
			// check if url does not contain root url. Base url itself contain root.
			// root is being checked because, admin side can use front end url.
			if(url.indexOf(jdvars['url']['root']) === -1) {
					// is it already routed URL without http ?
				  var base2_url_index = url.indexOf(jdvars['url']['base_without_scheme']);
				  // only add if, its not routed URL
				  if(base2_url_index > -1){
					 url = url.replace(jdvars['url']['base_without_scheme'], '');
				  }
				  
				  var containsSlash = url.substr(0,1);
				  if (containsSlash == '/') {
					  url = url.substr(1);
				  }
				  
				  url = jdvars['url']['base'] + url;
			}
			
			return url;
		}
	};

/*--------------------------------------------------------------
  Ajax related works
--------------------------------------------------------------*/

jd.ajax = {
		
	//jdFW_TODO : replace via jQuery code
	create : function(sParentId, sTag, sId){
		var objParent = this.$(sParentId);
		objElement = document.createElement(sTag);
		objElement.setAttribute('id',sId);
		if(objParent){
			objParent.appendChild(objElement);
		}
	},

	remove : function(sId){
		$(sId).remove();
	},
	
	default_error_callback : function (error){
		//jdFW_TODO : log to console
		alert("An error has occured\n"+error);
	},
	
	default_success_callback : function (result){
		
		//jdFW_TODO : log to console
		
		// we now have an array, that contains an array.
		for(var i=0; i<result.length;i++){

			var cmd 		= result[i][0];
			var id			= result[i][1];
			var property 	= result[i][2];
			var data 		= result[i][3];

			switch(cmd){
			case 'as': 	// assign or clear
				var objElement = $(id);
				if(objElement){
					if(property == 'innerHtml' || property == 'innerHTML'){
						$('#'+id).html(data);
					}else if(property == 'replaceWith'){
						$('#'+id).replaceWith(data);
					}else{
						eval("objElement."+property+"=  data \; ");
					}
				}

				break;

			case 'al':	// alert
				if(data){
					alert(data);}
				break;

			case 'ce':
				jd.ajax.create(id,property, data);
				break;

			case 'rm':
				jd.ajax.remove(id);
				break;

			case 'cs':	// call script
				var scr = id + '(';
				if($.isArray(data)){
					scr += '(data[0])';
					for (var l=1; l<data.length; l++) {
						scr += ',(data['+l+'])';
					}
				} else {
					scr += 'data';
				}
				scr += ');';
				eval(scr);
				break;

			default:
				alert("Unknow command: " + cmd);
			}
		}
	},

	error : function(Request, textStatus, errorThrown, errorCallback) {
		var message = '<strong>AJAX Loading Error</strong><br/>HTTP Status: '+Request.status+' ('+Request.statusText+')<br/>';
		message = message + 'Internal status: '+textStatus+'<br/>';
		message = message + 'XHR ReadyState: ' + Request.readyState + '<br/>';
		message = message + 'Raw server response:<br/>'+Request.responseText;
		errorCallback(message);	
	},
	
	success : function(msg, successCallback, errorCallback) 
	{
		var data = false;
		var exception = false;
		
		try {
			data = jd.ajax.junkFilter(msg);
		} catch(err) {
			data = err.message + "\n<br/>\n<pre>\n" + message + "\n</pre>";
			exception = true;
		}
		
		if (!data || exception){
			return errorCallback(data);
		}
		
		// Call the callback function
		return successCallback(data);
	},
	
	/**
	 * Invoke to clear data (remove warnings and error) from response
	 * response always availble betwwen '###' so we will propely formate it.
	 *   
	 */
	junkFilter : function( response_data) 
	{
		// Get rid of junk before the data
		var valid_pos = response_data.indexOf('###');
		var valid_last_pos = response_data.lastIndexOf('###');
		
		if( valid_pos == -1 ) {
			return false;
		}
		
		// get message between ###<----->### second argument is length
		meaningful_response_data = response_data.substr(valid_pos+3, valid_last_pos-(valid_pos+3)); 
		
		try {
			meaningful_response_data = $.parseJSON(meaningful_response_data);
			
		} catch(e) {
			console.log({'exception_was : ': e});
			return false;
		}
			
		return meaningful_response_data;
	}, 
	
	/*
	 * url : URL to call
	 * data : array / json / string / object
	 * */
	go : function (url, data, successCallback, errorCallback, timeout){
		// jdFW_TODO : If ajax not available, handle by Iframes
		
		// timeout 60 seconds
		if(timeout == null) timeout = 600000;
		if(errorCallback == null) errorCallback = jd.ajax.default_error_callback;
		if(successCallback == null) successCallback = jd.ajax.default_success_callback;

		// properly route the url
		ajax_url = jd.url.route(url) + '&format=ajax';
		
		var spinner_selector = false;
		// before ajax start, check any spinner selector availble or not if yes then on it
		if ( data && typeof data['spinner_selector'] != "undefined" &&  $(data['spinner_selector']).length > 0 ) {
			spinner_selector = data['spinner_selector'];
			// remove from data otherwise it will be post
			delete  data['spinner_selector'];
			//show spinner
			$(spinner_selector).show();
		}	
		
		//execute ajax
		// in jQ1.5+ first argument is url
		$.ajax(ajax_url, {
			type	: "POST",
			cache	: false,
			data	: data,
			timeout	: timeout,
			success	: function(msg) 
							{
								// stop spinner
								if ( spinner_selector ) {
									$(spinner_selector).hide();
								}
								
								jd.ajax.success(msg,successCallback,errorCallback);
							},
			error	: function(Request, textStatus, errorThrown)
							{
								// stop spinner
								if ( spinner_selector ) {
									$(spinner_selector).hide();
								}
								
								jd.ajax.error(Request, textStatus, errorThrown, errorCallback);
							}
		});
	}
};



jd.iframe = {

	show:function (call, appendTo,onLoadCallback){
		
		if(onLoadCallback == null) onLoadCallback = this.process;
		if(appendTo == null) appendTo = '#jdWindowContent';
		
		if(typeof call.classes === "undefined"){ 
			call.classes = '';
		} 
		
		if(typeof call.id === "undefined"){ 
			call.id = '';
		}
		
		$iframe = $('<iframe id="'+call.id+'" class="jd-iframe '+call.classes+'" frameborder="0" scrolling="auto" width="98%" height="90%" >');
		$iframe.load(onLoadCallback).appendTo(appendTo);
				
		// properly output the url
		url = jd.url.route(call.url);
		
		url += '&' + $.param(call.data);
		$iframe.attr('src',url);
		return $iframe;
	},
	
	process : function(){
		
	}
};


//Document ready
$(document).ready(function(){

	// load timeago
	$('.jd-timeago').timeago();
});

//ENDING :
//Scoping code for easy and non-conflicting access to $.
//Should be last line, write code above this line.
})(jd.jQuery);


/**
 * timeago: a jQuery plugin, version: 0.9.3 (2011-01-21)
 * @requires jQuery v1.2.3 or later
 *
 * Timeago is a jQuery plugin that makes it easy to support automatically
 * updating fuzzy timestamps (e.g. "4 minutes ago" or "about 1 day ago").
 *
 * For usage and examples, visit:
 * http://timeago.yarp.com/
 *
 * Licensed under the MIT:
 * http://www.opensource.org/licenses/mit-license.php
 *
 * Copyright (c) 2008-2011, Ryan McGeary (ryanonjavascript -[at]- mcgeary [*dot*] org)
 */
(function($) {
  $.timeago = function(timestamp) {
    if (timestamp instanceof Date) {
      return inWords(timestamp);
    } else if (typeof timestamp === "string") {
      return inWords($.timeago.parse(timestamp));
    } else {
      return inWords($.timeago.datetime(timestamp));
    }
  };
  var $t = $.timeago;

  $.extend($.timeago, {
    settings: {
      refreshMillis: 60000,
      allowFuture: true,
      strings: {
        prefixAgo: null,
        prefixFromNow: null,
        suffixAgo: "ago",
        suffixFromNow: "from now",
        seconds: "less than a minute",
        minute: "about a minute",
        minutes: "%d minutes",
        hour: "about an hour",
        hours: "about %d hours",
        day: "a day",
        days: "%d days",
        month: "about a month",
        months: "%d months",
        year: "about a year",
        years: "%d years",
        numbers: []
      }
    },
    inWords: function(distanceMillis) {
      var $l = this.settings.strings;
      var prefix = $l.prefixAgo;
      var suffix = $l.suffixAgo;
      if (this.settings.allowFuture) {
        if (distanceMillis < 0) {
          prefix = $l.prefixFromNow;
          suffix = $l.suffixFromNow;
        }
        distanceMillis = Math.abs(distanceMillis);
      }

      var seconds = distanceMillis / 1000;
      var minutes = seconds / 60;
      var hours = minutes / 60;
      var days = hours / 24;
      var years = days / 365;

      function substitute(stringOrFunction, number) {
        var string = $.isFunction(stringOrFunction) ? stringOrFunction(number, distanceMillis) : stringOrFunction;
        var value = ($l.numbers && $l.numbers[number]) || number;
        return string.replace(/%d/i, value);
      }

      var words = seconds < 45 && substitute($l.seconds, Math.round(seconds)) ||
        seconds < 90 && substitute($l.minute, 1) ||
        minutes < 45 && substitute($l.minutes, Math.round(minutes)) ||
        minutes < 90 && substitute($l.hour, 1) ||
        hours < 24 && substitute($l.hours, Math.round(hours)) ||
        hours < 48 && substitute($l.day, 1) ||
        days < 30 && substitute($l.days, Math.floor(days)) ||
        days < 60 && substitute($l.month, 1) ||
        days < 365 && substitute($l.months, Math.floor(days / 30)) ||
        years < 2 && substitute($l.year, 1) ||
        substitute($l.years, Math.floor(years));

      return $.trim([prefix, words, suffix].join(" "));
    },
    parse: function(iso8601) {
      var s = $.trim(iso8601);
      s = s.replace(/\.\d\d\d+/,""); // remove milliseconds
      s = s.replace(/-/,"/").replace(/-/,"/");
      s = s.replace(/T/," ").replace(/Z/," UTC");
      s = s.replace(/([\+\-]\d\d)\:?(\d\d)/," $1$2"); // -04:00 -> -0400
      return new Date(s);
    },
    datetime: function(elem) {
      // jQuery's `is()` doesn't play well with HTML5 in IE
      var isTime = $(elem).get(0).tagName.toLowerCase() === "time"; // $(elem).is("time");
      var iso8601 = isTime ? $(elem).attr("datetime") : $(elem).attr("title");
      return $t.parse(iso8601);
    }
  });

  $.fn.timeago = function() {
    var self = this;
    self.each(refresh);

    var $s = $t.settings;
    if ($s.refreshMillis > 0) {
      setInterval(function() { self.each(refresh); }, $s.refreshMillis);
    }
    return self;
  };

  function refresh() {
    var data = prepareData(this);
    if (!isNaN(data.datetime)) {
      $(this).text(inWords(data.datetime));
    }
    return this;
  }

  function prepareData(element) {
    element = $(element);
    if (!element.data("timeago")) {
      element.data("timeago", { datetime: $t.datetime(element) });
      var text = $.trim(element.text());
      if (text.length > 0) {
        element.attr("title", text + ' UTC');
      }
    }
    return element.data("timeago");
  }

  function inWords(date) {
    return $t.inWords(distance(date));
  }

  function distance(date) {
    return (new Date().getTime() - date.getTime());
  }

  // fix for IE6 suckage
  document.createElement("abbr");
  document.createElement("time");
}(jd.jQuery));