/**
 *
 * ############################################################################################
 * ######################### 		JD Credit Card namespace		 ##########################
 * ############################################################################################
 * 
 * 
 * Check Credit Card Validation 
 * Inspired by: https://github.com/stripe/jquery.payment
 *
 * @package     JDpkg
 * @since       1.1
 * 
 */
var JDcredit_card = function()
 { 
	var JDdefault_card_format = /(\d{1,4})/g; // Credit card format
	
	// Available Credit cards
	var cards = 
					[					 
	 					{
	 						type: 'visaelectron',
	 						pattern: /^4(026|17500|405|508|844|91[37])/,
	 						format: JDdefault_card_format,
	 						length: [16],
	 						cvcLength: [3],
	 						luhn: true
	 					}, {
	 						type: 'maestro',
	 						pattern: /^(5(018|0[23]|[68])|6(39|7))/,
	 						format: JDdefault_card_format,
	 						length: [12, 13, 14, 15, 16, 17, 18, 19],
	 						cvcLength: [3],
	 						luhn: true
	 					}, {
	 						type: 'forbrugsforeningen',
	 						pattern: /^600/,
	 						format: JDdefault_card_format,
	 						length: [16],
	 						cvcLength: [3],
	 						luhn: true
	 					}, {
	 						type: 'dankort',
	 						pattern: /^5019/,
	 						format: JDdefault_card_format,
	 						length: [16],
	 						cvcLength: [3],
	 						luhn: true
	 					}, {
	 						type: 'visa',
	 						pattern: /^4/,
	 						format: JDdefault_card_format,
	 						length: [13, 16],
	 						cvcLength: [3],
	 						luhn: true
	 					}, {
	 						type: 'mastercard',
	 						pattern: /^5[0-5]/,
	 						format: JDdefault_card_format,
	 						length: [16],
	 						cvcLength: [3],
	 						luhn: true
	 					}, {
	 						type: 'amex',
	 						pattern: /^3[47]/,
	 						format: /(\d{1,4})(\d{1,6})?(\d{1,5})?/,
	 						length: [15],
	 						cvcLength: [3, 4],
	 						luhn: true
	 					}, {
	 						type: 'dinersclub',
	 						pattern: /^3[0689]/,
	 						format: JDdefault_card_format,
	 						length: [14],
	 						cvcLength: [3],
	 						luhn: true
	 					}, {
	 						type: 'discover',
	 						pattern: /^6([045]|22)/,
	 						format: JDdefault_card_format,
	 						length: [16],
	 						cvcLength: [3],
	 						luhn: true
	 					}, {
	 						type: 'unionpay',
	 						pattern: /^(62|88)/,
	 						format: JDdefault_card_format,
	 						length: [16, 17, 18, 19],
	 						cvcLength: [3],
	 						luhn: false
	 					}, {
	 						type: 'jcb',
	 						pattern: /^35/,
	 						format: JDdefault_card_format,
	 						length: [16],
	 						cvcLength: [3],
	 						luhn: true
	 					}
	 					];
	/**
	 *  Invoke to get Card 
	 *  
	 * @param Numric Value 
	 * @returns card object if exist
	 */
	var getCard = 
			function( number)
			{
				var card, _i, _len;
				
				// \D :: replace any character that's not a digit [^0-9]
				// /g :: (modifier)global. All matches (don't return on first match)
				number = (number + '').replace(/\D/g, '');
				
				for (_i = 0, _len = cards.length; _i < _len; _i++) {
					card = cards[_i];
					if (card.pattern.test(number)) {
						return card;
					}
				}
				return null;
			},
			
			
	/**
	 * Invoke to get Card-type 
	 *  
	 * @param Numric Value 
	 * @returns card-type if exist
	 */			
	 getType = 
			function( number)
			{
				var _ref;
				if (!number) {
					return null;
				}
				
				return  (_ref = getCard(number)) != null ? _ref.type : null;
			},
	
	/**
	 * Invoke to check card-number is validate or not.
	 * 	luhn Checksum Algos apply
	 * 
	 * @param Nemeric Number 
	 * 
	 * @return (bool) true if valid number 
	 */
	isValidLuhn = function(num) 
			{
				 var digit, digits, odd, sum, _i, _len;
				 odd = true;
				 sum = 0;
				 digits = (num + '').split('').reverse();
				 
				 for (_i = 0, _len = digits.length; _i < _len; _i++) {
					 digit = digits[_i];
					 digit = parseInt(digit, 10);
					 
					 if ((odd = !odd)) {
						 digit *= 2;
					 }
					 
					 if (digit > 9) {
						 digit -= 9;
					 }
				 
					 sum += digit;
				 }
				 
				 return sum % 10 === 0;
			};
			
			
	return {
				getCard		:	getCard,	
				getType 	: 	getType,
				isValidLuhn	:	isValidLuhn,
			
			};
 }
	 			
/**
 * ############################################################################################
 * ########################### 		jd Form Validation		 ##################################
 * ############################################################################################
 * 
 * Unobtrusive Form Validation library
 * Inspired by: Chris Campbell <www.particletree.com>
 *
 * @package     JDpkg
 * @since       1.1
 */
var JDFormValidator = function() {
	var $, handlers, inputEmail, custom,

 	setHandler = function(name, fn, en) {
 	 	en = (en === '') ? true : en;
 	 	handlers[name] = {
 	 	 	enabled : en,
 	 	 	exec : fn
 	 	};
 	},

 	findLabel = function(id, form){
 	 	var $label, $form = jQuery(form);
 	 	if (!id) {
 	 	 	return false;
 	 	}
 	 	$label = $form.find('#' + id + '-lbl');
 	 	if ($label.length) {
 	 	 	return $label;
 	 	}
 	 	$label = $form.find('label[for="' + id + '"]');
 	 	if ($label.length) {
 	 	 	return $label;
 	 	}
 	 	return false;
 	},

 	handleResponse = function(state, $el, type, msg) {
 		
 		if ( $el.attr('data-jd-validate-error') ) {
 			var $error = $($el.attr('data-jd-validate-error'));
 			var $label = false;
 		} else if($el.attr('id')) {
 			var $label = $el.data('label');
 			var $error = $('[for="'+$el.attr('id')+'"]').not($label);
 		} else {
 			var $error = $('[for="'+$el.selector+'"]');
 			var $label = false; 
 		}
 		
 		type = typeof type !== 'undefined' ? type : 'error';
 		msg = typeof msg !== 'undefined' ? msg : $el.attr('error-message');
 		

 	 	// Set the element and its label (if exists) invalid state
 	 	if (state === false) {
 	 	 	$el.addClass('invalid').attr('aria-invalid', 'true'); 	 	 	
 	 	 	if($error){
 	 	 		$error.removeClass('hide');
 	 	 		$error.addClass('show').html(msg);
 	 	 	}
 	 	 	if($label){
 	 	 	 	$label.addClass('invalid').attr('aria-invalid', 'true');
 	 	 	}
 	 	} else {
 	 	 	$el.removeClass('invalid').attr('aria-invalid', 'false'); 	 	 	
 	 	 	if($error){
 	 	 		$error.removeClass('show');
 	 	 		$error.addClass('hide')
 	 	 	}
 	 	 	if ($label) {
 	 	 	 	$label.removeClass('invalid').attr('aria-invalid', 'false');
 	 	 	}
 	 	}
 	},

 	validate = function(el) {
 	 	var $el = jQuery(el), tagName, handler;
 	 	// Ignore the element if its currently disabled, because are not submitted for the http-request. For those case return always true.
 	 	if ($el.attr('disabled')) {
 	 	 	handleResponse(true, $el);
 	 	 	return true;
 	 	}
 	 	// If the field is required make sure it has a value
 	 	if ($el.attr('required') || $el.hasClass('required')) {
 	 	 	tagName = $el.prop("tagName").toLowerCase();
 	 	 	if (tagName === 'fieldset' && ($el.hasClass('radio') || $el.hasClass('checkboxes'))) {
 	 	 	 	if (!$el.find('input:checked').length){
 	 	 	 	 	handleResponse(false, $el);
 	 	 	 	 	return false;
 	 	 	 	}
 	 	 	//If element has class placeholder that means it is empty.
 	 	 	} else if (!$el.val() || $el.hasClass('placeholder') || ($el.attr('type') === 'checkbox' && !$el.is(':checked'))) {
 	 	 	 	handleResponse(false, $el);
 	 	 	 	return false;
 	 	 	}
 	 	}
 	 	// Only validate the field if the validate class is set
 	 	handler = ($el.attr('class') && $el.attr('class').match(/validate-([a-zA-Z0-9\_\-]+)/)) ? $el.attr('class').match(/validate-([a-zA-Z0-9\_\-]+)/)[1] : "";
 	 	if (handler === '') {
 	 	 	handleResponse(true, $el);
 	 	 	return true;
 	 	}
 	 	// Check the additional validation types
 	 	if ((handler) && (handler !== 'none') && (handlers[handler]) && $el.val()) {
 	 	 	// Execute the validation handler and return result
 	 	 	if (handlers[handler].exec($el, $el.val()) !== true) {
 	 	 	 	handleResponse(false, $el);
 	 	 	 	return false;
 	 	 	}
 	 	}
 	 	// Return validation state
 	 	handleResponse(true, $el);
 	 	return true;
 	},

 	isValid = function(form) {
 	 	var valid = true, i, message, errors, error, label;
 	 	// Validate form fields
 	 	jQuery.each(jQuery(form).find('input, textarea, select, fieldset, button'), function(index, el) {
 	 	 	if (($(el).is(':visible') || $(el).hasClass('validate-hidden'))&& validate(el) === false) {
 	 	 	 	valid = false;
 	 	 	}
 	 	});
 	 	// Run custom form validators if present
 	 	jQuery.each(custom, function(key, validator) {
 	 	 	if (validator.exec() !== true) {
 	 	 	 	valid = false;
 	 	 	}
 	 	});
 	 	if (!valid) {
 	 	 	//message = Joomla.JText._('JLIB_FORM_FIELD_INVALID');
 	 	 	
 	 	 	scrollToError(form);
 	 	 	
// 	 	 	Joomla.renderMessages(error);
 	 	}	 	
 	 	
 	 	return valid;
 	},
 	
 	scrollToError = function(form){
 		errors = jQuery(form).find("input.invalid, textarea.invalid, select.invalid, fieldset.invalid, button.invalid");
 	 	 	
 	 	var el = $(errors[0]);

 	 	if($(el).hasClass('validate-hidden')){
 	 		//if hidden element then calculate offset of its error element 	 		
 	 		var elOffset = $('[for="'+el.attr('id')+'"]').offset().top;
 	 	}else{
 	 		var elOffset = el.offset().top;
 	 	}
		
 	    var elHeight = el.height();
 	    var windowHeight = $(window).height();
 	    var offset;

 	    if (elHeight < windowHeight) {
 	    	offset = elOffset - ((windowHeight / 2) - (elHeight / 2));
 	    }
 	    else {
 	    	offset = elOffset;
 	    }
 	  
 	 	$('html, body').animate({
 	        scrollTop: offset
 	    }, 1000);
 	},
 	
 	attachToForm = function(form) {
 	 	var inputFields = [];
 	 	// Iterate through the form object and attach the validate method to all input fields.
 	 	$(form).find('input, textarea, select, fieldset, button').each(function() {
 	 	 	var $el = $(this), id = $el.attr('id'), tagName = $el.prop("tagName").toLowerCase();
 	 	 	if(!$el.is(':visible') && !$el.hasClass('validate-hidden')){
 	 	 		return true;
 	 	 	}
 	 	 	if ($el.hasClass('required')) {
 	 	 	 	$el.attr('aria-required', 'true').attr('required', 'required');
 	 	 	}
 	 	 	if ((tagName === 'input' || tagName === 'button') && $el.attr('type') === 'submit') {
 	 	 	 	if ($el.hasClass('validate')) {
 	 	 	 	 	$el.on('click', function() {
 	 	 	 	 	 	return isValid(form);
 	 	 	 	 	});
 	 	 	 	}
 	 	 	} else {
				//in case of chosen
 	 	 		if($el.hasClass('validate-hidden')){
 	 	 			$el.on('change', function() {
 	 	 	 	 	 	return validate(this);
 	 	 	 	 	});
 	 	 		}
 	 	 	 	if (tagName !== 'fieldset') {
 	 	 	 	 	$el.on('blur', function() {
 	 	 	 	 	 	return validate(this);
 	 	 	 	 	});
 	 	 	 	 	if ($el.hasClass('validate-email') && inputEmail) {
 	 	 	 	 	 	$el.get(0).type = 'email';
 	 	 	 	 	}
 	 	 	 	}
 	 	 	 	$el.data('label', findLabel(id, form));
 	 	 	 	inputFields.push($el);
 	 	 	}
 	 	});
 	 	$(form).data('inputfields', inputFields);
 	},

 	initialize = function(selector) {
 	 	$ = jQuery.noConflict();
 	 	handlers = {};
 	 	custom = custom || {};

 	 	inputEmail = (function() {
 	 	 	var input = document.createElement("input");
 	 	 	input.setAttribute("type", "email");
 	 	 	return input.type !== "text";
 	 	})();
 	 	// Default handlers
 	 	setHandler('username', function(element, value) {
 	 	 	regex = new RegExp("[\<|\>|\"|\'|\%|\;|\(|\)|\&]", "i");
 	 	 	return !regex.test(value);
 	 	});
 	 	setHandler('password', function(element, value) {
 	 	 	regex = /^\S[\S ]{2,98}\S$/;
 	 	 	return regex.test(value);
 	 	});
 	 	setHandler('numeric', function(element, value) {
 	 	 	regex = /^(\d|-)?(\d|,)*\.?\d*$/;
 	 	 	return regex.test(value);
 	 	});
 	 	setHandler('integer', function(element, value) {
 	 	 	regex = /^(\d|-)?(\d|,)*\d*$/;
 	 	 	return regex.test(value);
 	 	});
 	 	setHandler('email', function(element, value) {
 	 	 	regex = /^[a-zA-Z0-9.!#$%&‚Äô*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*$/;
 	 	 	return regex.test(value);
 	 	});
 	 	setHandler('image', function (element, value) {
			var imageSize  = 0;
			var fileField = element[0]; //as element is an object
			for (i = 0; i < fileField.files.length; i++){
				  //inputField.files[0].size gets the size of your file.
				  imageSize +=  fileField.files[i].size;
			}
			return (element.data('fileuploadlimit') > imageSize); 
	    });
		
		/*
		 * handler to validate minimum numeric value
		 * IMP: Element should have a minimum numeric value in data attribute 'data-jd-min'
		 */
 	 	setHandler('min', function (element, value) {
 	 		regex = /^(\d|-)?(\d|,)*\.?\d*$/;
			return (regex.test(value) && value > element.data('jd-min'));
	    });
 	 	
 	 	/*
 	 	 * Auto invoke when html element have 'validate-jd-credit-card' class
 	 	 *  Validation rule apply according to available creadit cards type 
 	 	 */
 	 	setHandler('jd-credit-card', function (element, value) {
 	 		
 	 		//clean value
 	 		value = (value + '').replace(/\D/g, '');
 	 		 
 	 		//Validation-1: get card is exist or not
 	 		var JDcard = new JDcredit_card();
 	 		
 	 		var card = JDcard.getCard(value);
 	 		
 	 		// Error : Card is not exist
 	 		if (!card) {
 	 			return false;
 	 		}
 	 		
 	 		//Validation-2 : Check card-number length
 	 		if( -1 == jd.jQuery.inArray(value.length, card.length) ) {
 	 			return false;
 	 		}

 	 		//Validation-3 : Checksum checking, Validate card number
 	 		if (!JDcard.isValidLuhn(value)) {
 	 			return false;
 	 		}
 	 		
			return true; 
	    });
 	 	
 	 	/*
 	 	 * Invoke to validate cvc length according to credit card
 	 	 * 	- Field must have data attribute "data-jd-validate", This is a creadit-card field.
 	 	 *  
 	 	 */
 	 	setHandler('jd-cvc-length', function (element, value) {
 	 		
 	 		var card_number,
 	 			card_selector = element.data('jd-validate'),	// get card selector
 	 			JDcard = new JDcredit_card();
 	 		
 	 		// get cart number
 	 		card_number = jd.jQuery(card_selector).val();
 	 		
 	 		// if card-number is not exit
 	 		if (!card_number) {
 	 			//error :: card is not exist
 	 			return true;
 	 		}
 	 		
 	 		//get card is exist or not
 	 		var card = JDcard.getCard(card_number);
 	 		
 	 		// Validation will not fire if card is not exist 
 	 		if (!card) { 
 	 			//error :: card is not valid 
 	 			return true;
 	 		}
 	 		
 	 		//Validation-1 : cvc length according to card cvc length
 	 		if( -1 == jd.jQuery.inArray(value.length, card.cvcLength) ) {
 	 			return false;
 	 		}
 	 		
 	 		return true; 
	    });
 	 	
 	 	/*
 	 	 *  Invoke to validate any field accoring to regex
 	 	 *  	- field must have regx patterm(data-jd-validate-pattern)
 	 	 *  	- like you want to field have mm/yy value then add data-jd-validate-pattern="^(0[1-9]|1[0-2])/([1-9][4-9])$" as field-attribute
 	 	 *  	
 	 	 *  
 	 	 */
 	 	setHandler('jd-regex-pattern', function (element, value) {
 	 		
 	 		var regex_pattern,  regex_pattern_string = element.data('validate-pattern');
 	 		
 	 		if (! regex_pattern_string ) {
 	 			// Addtribute "data-jd-validate-pattern" is not define
 	 			return true;
 	 		}
 	 		//get regex object
 	 		regex_pattern = new RegExp(regex_pattern_string );
 	 		
 	 		return regex_pattern.test(value); 
	    });
 	 	
 	 	/*
 	 	 *  Invoke to validate expiry date with current date 
 	 	 *  	- Two fields will use
 	 	 *  	- data-jd-validate-type attribute must be required for this validation
 	 	 *  	- data-jd-validate-type have either 'year' or 'month' value.
 	 	 *  	
 	 	 */
 	 	setHandler('jd-exp-date', function (element, value) {
 	 		
 	 		var month_number, year_number, date, mm, yyyy, month_el, year_el;
 	 		
 	 		var l = element.data('jd-validate-type');
 	 		
 	 		// element find who am I? (Either month-element or year-element )
 	 		if ( 'month' === element.data('jd-validate-type') ) {
 	 			month_el 	= 	element;
 	 			year_el		=	jd.jQuery(element.data('jd-validate'));
 	 		} else {
 	 			year_el 	= 	element;
 	 			month_el	=	jd.jQuery(element.data('jd-validate'));
 	 		}
 	 		
 	 		month_number	= month_el.val();
 	 		year_number		= year_el.val();
 	 		
 	 		if (!month_number && !year_number) {
 	 			return false 
 	 		}
 	 		
 	 		date = new Date();

 	 		mm 		= date.getMonth()+1; // start with 0 ie January is 0! so add 1
 	 		yyyy 	= date.getFullYear();

 	 		year_number	 = parseInt(year_number);
 	 		month_number = parseInt(month_number);
 	 		
 	 		// if input year-number is less than current year then return false 
 	 		if ( year_number < yyyy ) {
 	 			return false;
 	 		}
 	 		
 	 		// if curent year and input year number same then comapre month 
 	 		if (year_number == yyyy && month_number <= mm ) {
 	 			return false;
 	 		}

 	 		// if everything is ok then call manually handle-response
 	 		// (because both elements are diffrent, So we need manually invoke handle response )
 	 		handleResponse(true, month_el);
 	 		handleResponse(true, year_el);
 	 		
 	 		return true;
 	 		 
	    });
 	 	
 	 	// Attach to forms with class 'form-validate'
 	 	jQuery(selector).each(function() {
 	 	 	attachToForm(this);
 	 	}, this);
 	};
 
 	return {
 		initialize : initialize,
 	 	isValid : isValid,
 	 	validate : validate,
 	 	setHandler : setHandler,
 	 	attachToForm : attachToForm,
 	 	custom: custom,
 	 	handleResponse : handleResponse,
 	 	scrollToError : scrollToError
 	};
};

