SET FOREIGN_KEY_CHECKS = 0;

CREATE TABLE IF NOT EXISTS `#__joomdownloads_category` (
  `category_id` int(11) NOT NULL AUTO_INCREMENT,
  `title` varchar(255) NOT NULL,
  `alias` varchar(255) NOT NULL,
  `image` varchar(2083) DEFAULT NULL,
  `published` tinyint(1) NOT NULL DEFAULT '1',
  `ordering` int(11) NOT NULL,
  `description` text,
  `params` text,
  `created_date` datetime NOT NULL,
  `modified_date` datetime NOT NULL,
  PRIMARY KEY (`category_id`),
  KEY `idx_alias` (`alias`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `#__joomdownloads_product` (
  `product_id` int(11) NOT NULL AUTO_INCREMENT,
  `title` varchar(255) NOT NULL,
  `category_id` int(11) NOT NULL,
  `alias` varchar(255) NOT NULL,
  `published` tinyint(1) NOT NULL DEFAULT '1',
  `ordering` int(11) NOT NULL,
  `image` varchar(2083) DEFAULT NULL,
  `params` text,
  `hits` INT NULL DEFAULT '0', 
  `description` text NULL,
  `created_date` datetime NOT NULL,
  `modified_date` datetime NOT NULL,
  PRIMARY KEY (`product_id`),
  KEY `idx_alias` (`alias`),
  KEY `fkCPRCategory_id` (`category_id`),
  CONSTRAINT `fkCPRCategory_id` FOREIGN KEY (`category_id`) REFERENCES `#__joomdownloads_category` (`category_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `#__joomdownloads_release` (
  `release_id` int(11) NOT NULL AUTO_INCREMENT,
  `version` varchar(255) NOT NULL,
  `product_id` int(11) NOT NULL,
  `ordering` int(11) NOT NULL,
  `filename` varchar(255) DEFAULT NULL,
  `published` tinyint(1) NOT NULL,
  `params` text,
  `released_date` datetime NOT NULL,
  `modified_date` datetime NOT NULL,
  `created_date` datetime NOT NULL,
  PRIMARY KEY (`release_id`),
  KEY `idx_version` (`version`),
  KEY `fkCRLProduct_id` (`product_id`),
  CONSTRAINT `fkCRLProduct_id` FOREIGN KEY (`product_id`) REFERENCES `#__joomdownloads_product` (`product_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `#__joomdownloads_download` (
  `download_id` int(11) NOT NULL AUTO_INCREMENT,
  `product_id` int(11) NOT NULL,
  `release_id` int(11) NOT NULL,
  `user_id` int(11) NOT NULL,
  `created_date` datetime NOT NULL,
  PRIMARY KEY (`download_id`),
  KEY `idx_user_id` (`user_id`),
  KEY `fkCDWRelease_id` (`release_id`),
  KEY `fkCDWProduct_id` (`product_id`),
  CONSTRAINT `fkCDWRelease_id` FOREIGN KEY (`release_id`) REFERENCES `#__joomdownloads_release` (`release_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `fkCDWProduct_id` FOREIGN KEY (`product_id`) REFERENCES `#__joomdownloads_product` (`product_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `#__joomdownloads_access` (
  `release_id` int(11) NOT NULL,
  `resource_type` varchar(255) NOT NULL,
  `resource_id` int(11) NOT NULL,
  PRIMARY KEY (`release_id`,`resource_type`,`resource_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `#__joomdownloads_field` (
  `field_id` int(11) NOT NULL AUTO_INCREMENT,
  `title` varchar(100) NOT NULL,
  `type` varchar(50) NOT NULL,
  `css_class` varchar(100) DEFAULT NULL,
  `filterable` tinyint(1) NOT NULL,
  `searchable` tinyint(1) DEFAULT '0',
  `published` tinyint(1) NOT NULL DEFAULT '1',
  `config` text,
  `ordering` int(11) DEFAULT '0',
  PRIMARY KEY (`field_id`),
  KEY `idx_type` (`type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `#__joomdownloads_fieldoption` (
  `fieldoption_id` int(11) NOT NULL AUTO_INCREMENT,
  `title` varchar(100) NOT NULL,
  `field_id` int(11) NOT NULL,
  `option_ordering` int(11) NOT NULL,
  PRIMARY KEY (`fieldoption_id`),
  KEY `fkCDFOField_id` (`field_id`),
  CONSTRAINT `fkCDFOField_id` FOREIGN KEY (`field_id`) REFERENCES `#__joomdownloads_field` (`field_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `#__joomdownloads_fieldposition` (
  `category_id` int(11) NOT NULL,
  `field_id` int(11) NOT NULL,
  `position` varchar(25) NOT NULL,
  `field_ordering` int(11) NOT NULL,
  PRIMARY KEY (`category_id`, `field_id`, `position`),
  KEY `fkCDFPField_id` (`field_id`),
  KEY `fkCDFPCategory_id` (`category_id`),
  CONSTRAINT `fkCDFPField_id` FOREIGN KEY (`field_id`) REFERENCES `#__joomdownloads_field` (`field_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `fkCDFPCategory_id` FOREIGN KEY (`category_id`) REFERENCES `#__joomdownloads_category` (`category_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `#__joomdownloads_fieldvalue` (
  `product_id` int(11) NOT NULL,
  `field_id` int(11) NOT NULL,
  `value` varchar(500) NOT NULL,
  KEY `fkCDFVField_id` (`field_id`),
  KEY `fkCDFVProduct_id` (`product_id`),
  KEY `idx_value` (`value`),
  CONSTRAINT `fkCDFVField_id` FOREIGN KEY (`field_id`) REFERENCES `#__joomdownloads_field` (`field_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `fkCDFVProduct_id` FOREIGN KEY (`product_id`) REFERENCES `#__joomdownloads_product` (`product_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

SET FOREIGN_KEY_CHECKS = 1;