<?php

/**
* @copyright	Copyright (C) 2015 Codextrous . All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package 		JOOMDOWNLOADS
* @subpackage	Front-end
* @contact		contact@codextrous.com
*/

// no direct access
defined('_JEXEC') or die;

/** 
 * Product Controller
 * @author Gaurav Jain
 */
class JOOMDownloadsSiteControllerProduct extends JOOMDownloadsController
{
	public function display($cachable = false, $urlparams = array())
	{
		$id = $this->_getId();
		
		if(empty($id)){
			throw new Exception("Product not found.", 404);
		}

		$product = $this->getModel()->loadRecords(array('product_id' => $id));
		if(empty($product)){
			throw new Exception("Product not found.", 404);
		}

		$product = array_shift($product);
		if($product->published == false){
			throw new Exception("Product is not published.", 404);	
		}
		
		$this->getView()->assign('product_id', $product->product_id);
		
		$productIns = JOOMDownloadsProduct::getInstance($product->product_id);
		$tempData 	= $productIns->toArray();
		$productIns->bind(array('hits' => $tempData['hits'] + 1))->save();

		return true;
	}

	public function download()
	{
		$release_id = $this->input->get('release_id', 0);
		$invalid = false;
		if(!$release_id){
			$invalid = true;
		}

		if(!$invalid){
			try{
				$release = JOOMDownloadsRelease::getInstance($release_id);
				$releaseData = $release->toArray();
				if(!$releaseData['published']){
					$invalid = true;
				}
			}
			catch(Exception $e){
				$invalid = true;
			}
		}

		if($invalid){			
			throw new Exception(JText::_('COM_JOOMDOWNLOADS_FILE_NOT_FOUND'), 404);
			return false;
		}

		$release = JOOMDownloadsRelease::getInstance($release_id);
		$releaseData = $release->toArray();

		$product      = JOOMDownloadsProduct::getInstance($releaseData['product_id'])->toArray();
		$productUrl = JRoute::_(JOOMDownloadsHelperRoute::getProductRoute($releaseData['product_id'], $product['category_id']));
		
		$accessData   = JOOMDownloadsFactory::getInstance('release', 'model')->getAccessRecords($release_id);

		JDHelperJoomla::loadPlugins('joomdownloads');
		$args         = array($product, $accessData);
		$pluginResult = JDHelperJoomla::triggerPlugin('onJoomdownloadsProductDownload',$args, '', $this);
		if(in_array(false, $pluginResult)){			
			$this->setRedirect($productUrl, JText::_('COM_JOOMDOWNLOADS_DOWNLOAD_NOT_AUTORIZED'), 'error');
			return false;
		}
		
		
		$file = $release->getDownloadFile();
		if(!JFile::exists($file)){
			$this->setRedirect($productUrl, JText::_('COM_JOOMDOWNLOADS_FILE_NOT_FOUND'), 'error');
			return false;
		}

		$release->addDownload();

		$known_mime_types	= JOOMDownloadsFactory::getHelper('utils')->getAllowedMimeTypes();
		       
        $file_extension = strtolower(substr(strrchr($file,"."),1));
        if(array_key_exists($file_extension, $known_mime_types)){
           $mime_type=$known_mime_types[$file_extension];
        } else {
           $mime_type="application/force-download";
        };
		  
		@ob_end_clean(); //turn off output buffering to decrease cpu usage
		  
		// required for IE, otherwise Content-Disposition may be ignored
		if(ini_get('zlib.output_compression'))
		 ini_set('zlib.output_compression', 'Off');
		  
		header('Content-Type: ' . $mime_type);
		header("Content-Disposition: attachment; filename=".$product['alias'].'-'.$releaseData['version'].'.'.JFile::getExt($file));
		header("Content-Transfer-Encoding: binary");
		header('Accept-Ranges: bytes');
		  
		 /* The three lines below basically make the 
		    download non-cacheable */
		header("Cache-control: private");
		header('Pragma: private');
		header("Expires: Mon, 26 Jul 1997 05:00:00 GMT"); 

		echo file_get_contents($file);
		exit();
	}
	
	public function releases()
	{
		$id = $this->_getId();
		
		if(empty($id)){
			throw new Exception("Product not found.", 404);
		}
		
		$product = $this->getModel()->loadRecords(array('product_id' => $id));
		if(empty($product)){
			throw new Exception("Product not found.", 404);
		}

		$product = array_shift($product);
		if($product->published == false){
			throw new Exception("Product is not published.", 404);	
		}
		
		$this->getView()->assign('product_id', $product->product_id);
		return true;
	}
}