<?php

/**
* @copyright	Copyright (C) 2015 Codextrous . All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package 		JOOMDOWNLOADS
* @subpackage	Back-end
* @contact		contact@codextrous.com
*/

// no direct access
defined('_JEXEC') or die;

class JOOMDownloadsHelperUtils extends JObject
{
	function saveUploadedFile($storagepath='', $filepath='', $filename='')
	{
		//no file selected then do nothing
		if(empty($filepath))
		{
		  	return false;
		}
	
		$app = JFactory::getApplication();
	
		//remove backslashes from file name
		$filename = JFile::makesafe($filename);

		//check if folder exist or not. If not exists then create it.
		if(JFolder::exists($storagepath)==false){
			JFolder::create($storagepath);
		}
		
		if(copy($filepath, $storagepath.$filename)){
			return $filename;	
		}
		
		return false;		
	}

	public function getReturnUrl()
	{
		$app	= JFactory::getApplication();
		$router = $app->getRouter();
		// Stay on the same page
		$uri = clone JURI::getInstance();
		$vars = $router->parse($uri);
		unset($vars['lang']);
		if ($router->getMode() == JROUTER_MODE_SEF)
		{
			if (isset($vars['Itemid']))
			{
				$itemid = $vars['Itemid'];
				$menu = $app->getMenu();
				$item = $menu->getItem($itemid);
				unset($vars['Itemid']);
				if (isset($item) && $vars == $item->query)
				{
					$url = 'index.php?Itemid='.$itemid;
				}
				else {
					$url = 'index.php?'.JURI::buildQuery($vars).'&Itemid='.$itemid;
				}
			}
			else
			{
				$url = 'index.php?'.JURI::buildQuery($vars);
			}
		}
		else
		{
			$url = 'index.php?'.JURI::buildQuery($vars);
		}		

		return base64_encode($url);
	}

	public function getAllowedMimeTypes()
	{
		return array(
			'pdf' => 'application/pdf' ,
			'ppt' => 'application/powerpoint' ,
			'gz' => 'application/x-gzip' ,
			'tar' => 'application/x-tar' ,
			'tgz' => 'application/x-tar' ,
			'zip' => array('application/x-zip', 'application/zip') ,
			'bmp' => 'image/bmp' ,
			'gif' => 'image/gif' ,
			'jpeg' => 'image/jpeg' ,
			'jpg' => 'image/jpeg' ,
			'jpe' => 'image/jpeg' ,
			'png' => 'image/png' ,
			'tiff' => 'image/tiff' ,
			'tif' => 'image/tiff' ,
			'txt' => 'text/plain' ,
			'mpeg' => 'video/mpeg' ,
			'mpg' => 'video/mpeg' ,
			'mpe' => 'video/mpeg' ,
			'qt' => 'video/quicktime' ,
			'mov' => 'video/quicktime' ,
			'avi' => 'video/x-msvideo' ,
			'flv' => 'video/x-flv' ,
			'doc' => 'application/msword' ,
			'mp4' => array('video/mp4' , 'application/octet-stream')
		);
	}

	public function getHTMLTagsUpload() {
		return array('abbr','acronym','address','applet','area','audioscope','base','basefont','bdo','bgsound','big','blackface','blink','blockquote','body','bq','br','button','caption','center','cite','code','col','colgroup','comment','custom','dd','del','dfn','dir','div','dl','dt','em','embed','fieldset','fn','font','form','frame','frameset','h1','h2','h3','h4','h5','h6','head','hr','html','iframe','ilayer','img','input','ins','isindex','keygen','kbd','label','layer','legend','li','limittext','link','listing','map','marquee','menu','meta','multicol','nobr','noembed','noframes','noscript','nosmartquotes','object','ol','optgroup','option','param','plaintext','pre','rt','ruby','s','samp','script','select','server','shadow','sidebar','small','spacer','span','strike','strong','style','sub','sup','table','tbody','td','textarea','tfoot','th','thead','title','tr','tt','ul','var','wbr','xml','xmp','!DOCTYPE', '!--');
	}

	public function canUpload( $file, &$err) 
	{
		$allowedMimeType = self::getAllowedMimeTypes();
		
		$upload_extensions 	= array_keys($allowedMimeType);
		$image_extensions 	= 'bmp,gif,jpg,png,jpeg';
		$upload_mime		= $allowedMimeType;
		
		// The file doesn't exist
		if(empty($file['name'])) {
			$err = 'COM_JOOMDOWNLOADS_FILE_UPLOAD_NAME_MISSING';
			return false;
		}
		// Not safe file
		jimport('joomla.filesystem.file');
		if ($file['name'] !== JFile::makesafe($file['name'])) {
			$err = 'COM_JOOMDOWNLOADS_FILE_INVALID';
			return false;
		}

		$format 	= strtolower(JFile::getExt($file['name']));
		if ($format == '' || $format == false || (!in_array($format, $upload_extensions))) {
			$err = 'COM_JOOMDOWNLOADS_FILE_TYPE_NOT_ALLOWED';
			return false;
		}
		
		// Max size of image
		$maxSize = 3145728; // 3MB
		if ((int)$maxSize > 0 && (int)$file['size'] > (int)$maxSize) {
			$err = 'COM_JOOMDOWNLOADS_FILE_TOO_LARGE';				
			return false;
		}

		// Image check
		$imginfo	= null;
		$images		= explode( ',', $image_extensions);
		
		if(in_array($format, $images)) { // if its an image run it through getimagesize			
			if(($imginfo = getimagesize($file['tmp_name'])) === FALSE) {
				$err = 'COM_JOOMDOWNLOADS_FILE_INVALID_IMAGE';
				$err = $imginfo[0];
				return false;
			}
		} else if(!in_array($format, $images)) { // if its not an image...and we're not ignoring it	
			$allowed_mime = $upload_mime;
			if(function_exists('finfo_open')) {// We have fileinfo
				$finfo	= finfo_open(FILEINFO_MIME);
				$type	= finfo_file($finfo, $file['tmp_name']);
				$type 	= explode(';', $type);
				$type 	= array_shift($type);

                if(!isset($allowed_mime[$format])
                    || (!is_array($allowed_mime[$format]) && $allowed_mime[$format] != $type)
                    || (is_array($allowed_mime[$format]) && !in_array($type, $allowed_mime[$format])))
                {
					$err = 'COM_JOOMDOWNLOADS_FILE_INVALID_MIME';
					return false;
				}
				finfo_close($finfo);
			} else if(function_exists('mime_content_type')) { // we have mime magic
				$type = mime_content_type($file['tmp_name']);
				if(strlen($type) && !in_array($type, $allowed_mime)) {
					$err = 'COM_JOOMDOWNLOADS_FILE_INVALID_MIME';
					return false;
				}
			}
		}
			
		// XSS Check
		$xss_check =  JFile::read($file['tmp_name'],false,256);
		$html_tags = self::getHTMLTagsUpload();
		foreach($html_tags as $tag) { // A tag is '<tagname ', so we need to add < and a space or '<tagname>'
			if(stristr($xss_check, '<'.$tag.' ') || stristr($xss_check, '<'.$tag.'>')) {
				$err = 'COM_JOOMDOWNLOADS_FILE_XSS';
				return false;
			}
		}
		
		return true;
	}

	public function getDayUnix($date){
		$date 	 = JDDate::getInstance($date);
		$year 	 = $date->toFormat('Y');
		$month 	 = $date->toFormat('m');
		$day 	 = $date->toFormat('d');
		$date    = new JDDate(mktime(0,0,0,$month,$day,$year));
		return $date->toUnix();
	}
}
