<?php

/**
* @copyright	Copyright (C) 2015 Codextrous . All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package 		JOOMDOWNLOADS
* @subpackage	Back-end
* @contact		contact@codextrous.com
*/

// no direct access
defined('_JEXEC') or die;

/** 
 * Field Model
 * @author Gaurav Jain
 */
class JOOMDownloadsModelField extends JOOMDownloadsModel
{
	/**
	 * (non-PHPdoc)
	 * @see plugins/system/jd/jd/jd/JDModel::_buildWhereClause()
	 */
	protected function _buildWhereClause(JDQuery $query, Array $queryFilters)
	{
		foreach($queryFilters as $key=>$value){
			//support id too, replace with actual name of key
			$key = ($key==='id')? $this->getTable()->getKeyName() : $key;
			
			// only one condition for this key
			if(!is_array($value)){
				$query->where("`tbl`.`$key` =".$this->_db->Quote($value));
				continue;
			}
			
			// multiple keys are there
			foreach($value as $condition){
				// not properly formatted
				if(is_array($condition)==false){
					continue;
				}
				// first value is condition, second one is value
				$glue = 'AND';
				list($operator, $val)= $condition;
				
				if (3 == count($condition)) {
					list($operator, $val, $glue)= $condition;
				}
				
				$query->where("`tbl`.`$key` $operator ".$val, $glue);
			}
		}
	}
	
}

class JOOMDownloadsModelformField extends JOOMDownloadsModelform {
	public function getControlNamePrefix()
	{
		return $this->_component->getNameSmall().'_'.$this->getName().'_form';
	}
}

class JOOMDownloadsModelFieldOption extends JOOMDownloadsModel{
	/**
	 * Load options of the given fieldId
	 *
	 * @param $fieldId : id for which options to be loaded
	 * @param $optionIds(optional) : option ids for which data to be loaded,
	 * 								 otherwise all the options of current fieldId will be loaded 
	 *
	 * @return array of resultant rows
	 */
	function loadOptions($fieldId, Array $optionIds = array(), $indexBy = 'fieldoption_id')
	{
		$query = new JDQuery();
		
		if(!empty($optionIds)){
			$query->where('ao.fieldoption_id IN('.implode(',', $optionIds).')');
		}
		
		return $query->select('*')
		 		     ->from('#__joomdownloads_fieldoption as ao')
		 		     ->where('ao.field_id = '.$fieldId)
		 		     ->order('ao.option_ordering')
		 		     ->dbLoadQuery()
		 		     ->loadAssocList($indexBy);
	}
	
	/**
	 * delete options data from both option and option_lang table
	 */
	function deleteOptions($fieldId, $optionId)
	{
		$query = new JDQuery();
		
		if(!is_null($optionId)){
			$query->where('b.fieldoption_id = '. $optionId);
		}
		if(!is_null($fieldId)){
			$query->where('b.field_id = '. $fieldId);
		}	
		
		//Due to some limitation of joomla's delete function, here we used jd_delete to add elements
		return $query->multiDelete(null, 'a.*, b.*')
					 ->from('`#__joomdownloads_fieldoption` as b')
					 ->where('a.fieldoption_id = b.fieldoption_id')
					 ->dbLoadQuery()
					 ->query();
	}
}