<?php

/**
* @copyright	Copyright (C) 2015 Codextrous . All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package 		JOOMDOWNLOADS
* @subpackage	Back-end
* @contact		contact@codextrous.com
*/

// no direct access
defined('_JEXEC') or die;

/** 
 * Product Model
 * @author Gaurav Jain
 */
class JOOMDownloadsModelProduct extends JOOMDownloadsModel
{
	public $filterMatchOpeartor = array(
		'title' => array('LIKE'),
		'category_id' => array('='),
		'published' => array('='));

	public function getCategories()
	{
		$db = $this->_db;
		$query = $db->getQuery(true);
		$query->select('DISTINCT `category`')
				->from('`#__joomdownloads_product`');

		$db->setQuery($query);
		return $db->loadObjectList();
	}

	public function deleteFieldValues($productId)
	{
		$db = $this->_db;
		$query = $db->getQuery(true);
		$query->delete()
				->from('`#__joomdownloads_fieldvalue`')
				->where('`product_id` = '.$db->quote($productId));

		$db->setQuery($query);
		return $db->query();
	}

	public function insertFieldValues($productId, $fieldData)
	{
		$db = $this->_db;
		$insert = array();
		foreach($fieldData as $fieldId => $data){
			if(empty($data)){
				continue;
			}

			if(!is_array($data)){
				$insert[] = '('.$db->quote($productId).', '.$db->quote($fieldId).', '.$db->quote($data).')';
				continue;
			}
			foreach($data as $value){
				$insert[] = '('.$db->quote($productId).', '.$db->quote($fieldId).', '.$db->quote($value).')';
			}
		}

		if(empty($insert)){
			return true;
		}
		
		$db = $this->_db;
		$query = 'INSERT INTO `#__joomdownloads_fieldvalue` (`product_id`, `field_id`, `value`) VALUES '.
				 implode(',', $insert);
				 
		$db->setQuery($query);
		return $db->query();
	}

	public function getFieldValues($productId)
	{
		$db = $this->_db;
		$query = $db->getQuery(true);
		$query->select('*')
				->from('`#__joomdownloads_fieldvalue`')
				->where('`product_id` = '.$db->quote($productId));

		$db->setQuery($query);
		$records = $db->loadObjectList();

		$fieldValues = array();
		foreach($records as $record){
			if(!isset($fieldValues[$record->field_id])){
				$fieldValues[$record->field_id] = $record->value;
				continue;
			}

			if(!is_array($fieldValues[$record->field_id])){
				$fieldValues[$record->field_id] = array($fieldValues[$record->field_id], $record->value);
				continue;			
			}

			$fieldValues[$record->field_id][] = $record->value;
		}

		return $fieldValues;
	}

	public function getAllAccessRecords()
	{
		static $accessData = null;

		if($accessData !== null){
			return $accessData;
		}
		
		$db = $this->_db;
		$query = $db->getQuery(true);
		$query->select('*')
				->from('`#__joomdownloads_access`');

		$db->setQuery($query);
		$records = $db->loadObjectList();

		$accessData = array();
		foreach($records as $record){
			if(!isset($accessData[$record->resource_type])){
				$accessData[$record->resource_type] = array();
			}

			if(!isset($accessData[$record->resource_type][$record->product_id])){
				$accessData[$record->resource_type][$record->product_id]['resource_id'] = $record->resource_id;
				continue;
			}

			if(!is_array($accessData[$record->resource_type][$record->product_id]['resource_id'])){
				$accessData[$record->resource_type][$record->product_id]['resource_id'] = array($accessData[$record->resource_type][$record->product_id]['resource_id'], $record->resource_id);
				continue;			
			}

			$accessData[$record->resource_type][$record->product_id]['resource_id'][] = $record->resource_id;
		}

		return $accessData;
	}
	
	public function getRelease($product_id)
	{
		if (empty($product_id)){
			return true;
		}
		
		$db = $this->_db;
		$sql = ' SELECT r.release_id, r.product_id, r.version, p.category_id, p.title FROM #__joomdownloads_product p, #__joomdownloads_release r 
			 	 WHERE p.product_id = r.product_id AND r.published = 1 AND p.published = 1
				 AND r.product_id ';
		
		if (is_array($product_id)){
			$sql .= ' IN ('.implode(',', $product_id).')';
		}else{
			$sql .= ' = '.$product_id;
		}
		
		$db->setQuery($sql);
		$records = $db->loadObjectList();
		
		return $records;
	}
}

class JOOMDownloadsModelformProduct extends JOOMDownloadsModelform { }