<?php
/**
 * @package     JD.Site
 * @subpackage  com_joomdownloads
 *
 * @copyright   Copyright (C) 20015 - 2016 Codextrous Software Private Limited
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

/**
 * Routing class from com_joomdownloads
 *
 * @since  3.3
 */
class JoomdownloadsRouter extends JComponentRouterBase
{
	protected static $language_loaded = false;
	
	protected function _loadLanguage()
	{
		if (static::$language_loaded == false){
			$lang = JFactory::getLanguage();
			$lang->load('com_joomdownloads', JPATH_SITE, $lang->getTag(), true);
			static::$language_loaded = true;
		}
	}

    /**
     * Build the route for the com_joomdownloads component
     *
     * @param   array  &$query  An array of URL arguments
     *
     * @return  array  The URL arguments to use to assemble the subsequent URL.
     *
     * @since   3.3
     */
    public function build(&$query)
    {
        $segments = array();

        // Unset limitstart=0 since it's pointless
        if (isset($query['limitstart']) && $query['limitstart'] == 0)
        {
            unset($query['limitstart']);
        }

        // We need a menu item.  Either the one specified in the query, or the current active one if none specified
        if (empty($query['Itemid']))
        {
            $menuItem = $this->menu->getActive();
            $menuItemGiven = false;
        }
        else
        {
            $menuItem = $this->menu->getItem($query['Itemid']);
            $menuItemGiven = true;
        }

        // Check again
        if ($menuItemGiven && isset($menuItem) && $menuItem->component != 'com_joomdownloads')
        {
            $menuItemGiven = false;
            unset($query['Itemid']);
        }

        if (isset($query['view']))
        {
            $view = $query['view'];
        }
        else
        {
            // We need to have a view in the query or it is an invalid URL
            return $segments;
        }

        // Are we dealing with an product or category that is attached to a menu item?
        if (($menuItem instanceof stdClass)
            && $menuItem->query['view'] == $query['view']
            && isset($query['id'])
            && $menuItem->query['id'] == (int) $query['id'])
        {
            unset($query['view']);

            if (isset($query['catid']))
            {
                unset($query['catid']);
            }           

            if(isset($query['task']) && $query['task'] == 'download'){
            	$this->_loadLanguage();
                 $release_id = $query['release_id'];

                $db = JFactory::getDbo();
                $dbQuery = $db->getQuery(true)
                    ->select('version')
                    ->from('#__joomdownloads_release')
                    ->where('release_id=' . (int) $query['release_id']);
                $db->setQuery($dbQuery);
                $version = $db->loadResult();
                $query['release_id'] = $version;

                $segments[] = $version;
              	$segments[] = JText::_('COM_JOOMDOWNLOADS_TASK_DOWNLOAD');

                unset($query['task']);
                unset($query['release_id']);
            }
            
       		if(isset($query['task']) && $query['task'] == 'releases'){
       			$this->_loadLanguage();
             	$segments[] = JText::_('COM_JOOMDOWNLOADS_TASK_RELEASES');
                unset($query['task']);
            }

            unset($query['id']);
            return $segments;
        }

        if ($view == 'category' || $view == 'product')
        {
            if (!$menuItemGiven)
            {
                $segments[] = $view;
            }

            unset($query['view']);

            if ($view == 'product')
            {
                if (isset($query['id']) && isset($query['catid']) && $query['catid'])
                {
                    $catid = $query['catid'];

                    $db = JFactory::getDbo();
                    $dbQuery = $db->getQuery(true)
                        ->select('alias')
                        ->from('#__joomdownloads_product')
                        ->where('product_id=' . (int) $query['id']);
                    $db->setQuery($dbQuery);
                    $alias = $db->loadResult();
                    $query['id'] = $alias;
                }
                else
                {
                    // We should have these two set for this view.  If we don't, it is an error
                    return $segments;
                }

                if(isset($query['task']) && $query['task'] == 'download' && isset($query['release_id'])){
                    $release_id = $query['release_id'];

                    $db = JFactory::getDbo();
                    $dbQuery = $db->getQuery(true)
                        ->select('version')
                        ->from('#__joomdownloads_release')
                        ->where('release_id=' . (int) $query['release_id']);
                    $db->setQuery($dbQuery);
                    $version = $db->loadResult();
                    $query['release_id'] = $version;
                }
            }
            else
            {
                if (isset($query['id']))
                {
                    $catid = $query['id'];
                }
                else
                {
                    // We should have id set for this view.  If we don't, it is an error
                    return $segments;
                }
            }

            if ($menuItemGiven && isset($menuItem->query['id']))
            {
                $mCatid = $menuItem->query['id'];
            }
            else
            {
                $mCatid = 0;
            }

            $categories = self::getCategories();
            $category = $categories[$catid];

            if (!$category)
            {
                // We couldn't find the category we were given.  Bail.
                return $segments;
            }

            $array = array();
            if ((int) $category->category_id != (int) $mCatid)
            {
                $array = array($category->alias);
            }

            $segments = array_merge($segments, $array);

            if ($view == 'product')
            {
                $segments[] = $query['id'];

                if(isset($query['task']) && $query['task'] == 'download'){
                	$this->_loadLanguage();   
                    $segments[] = $query['release_id'];
                    $segments = JText::_('COM_JOOMDOWNLOADS_TASK_DOWNLOAD'); 
                    unset($query['task']);
                    unset($query['release_id']);
                }
                
            if(isset($query['task']) && $query['task'] == 'releases'){      
            		$this->_loadLanguage();              
                    $segments[] = JText::_('COM_JOOMDOWNLOADS_TASK_RELEASES');
                    unset($query['task']);
                }
            }

            unset($query['id']);
            unset($query['catid']);
        }

        $total = count($segments);

        for ($i = 0; $i < $total; $i++)
        {
            $segments[$i] = str_replace(':', '-', $segments[$i]);
        }

        return $segments;
    }

    /**
     * Parse the segments of a URL.
     *
     * @param   array  &$segments  The segments of the URL to parse.
     *
     * @return  array  The URL attributes to be used by the application.
     *
     * @since   3.3
     */
    public function parse(&$segments)
    {
        $total = count($segments);
        $vars = array();

        // Get the active menu item.
        $item = $this->menu->getActive();
        $db = JFactory::getDbo();

        // Count route segments
        $count = count($segments);

        if(!isset($item)){
            $view = $segments[0];
            $alias = $segments[1];
        }
        else{
            if('category' == $item->query['view']){
                $view = 'category';
                if(count($segments) > 0){
                    $view = 'product';    
                }
                $alias  = $segments[0];

                $this->_loadLanguage();
                if(count($segments) > 1 && ((count($segments) < 3  || $segments[$count - 1] != JText::_('COM_JOOMDOWNLOADS_TASK_DOWNLOAD')) && (count($segments) < 2 || $segments[$count - 1] != JText::_('COM_JOOMDOWNLOADS_TASK_RELEASES')))){
                    JError::raiseError(404, JText::_('COM_JOOMDOWNLOADS_UNEXPECTED_ROUTE'));
                }
            }
            else if('product' == $item->query['view']){
                $view   = $item->query['view'];                
                $product_id = $item->query['id'];
                $this->_loadLanguage();
                if(count($segments) && ((count($segments) < 2  || $segments[$count - 1] != JText::_('COM_JOOMDOWNLOADS_TASK_DOWNLOAD')) && (count($segments) < 1  || $segments[$count - 1] != JText::_('COM_JOOMDOWNLOADS_TASK_RELEASES')))){
                    JError::raiseError(404, JText::_('COM_JOOMDOWNLOADS_UNEXPECTED_ROUTE'));
                }
            }
        }

        if($view == 'category'){
            // First we check if it is a category
            $query = $db->getQuery(true)
                    ->select($db->quoteName(array('category_id', 'alias')))
                    ->from($db->quoteName('#__joomdownloads_category'))
                    ->where($db->quoteName('alias') . ' = ' . $db->quote($alias));
            $db->setQuery($query);
            $category = $db->loadObject();

            if ($category && $category->alias == $alias)
            {
                $vars['view'] = 'category';
                $vars['id'] = $category->category_id;

                return $vars;
            }
        }
        else if($view == 'product')
        {
            $query = $db->getQuery(true)
                        ->select($db->quoteName(array('product_id', 'alias', 'category_id')))
                        ->from($db->quoteName('#__joomdownloads_product'));

            if(isset($product_id)){
                $query->where($db->quoteName('product_id') . ' = ' . $db->quote($product_id));
            }
            else{
                $query->where($db->quoteName('alias') . ' LIKE ' . $db->quote($alias));
            }

            $db->setQuery($query);
            $product = $db->loadObject();

            if ($product)
            {
                if ( (isset($product_id) && $product->product_id == $product_id) || (isset($alias) && $product->alias == $alias))
                {
                    $vars['view'] = 'product';
                    $vars['catid'] = (int) $product->category_id;
                    $vars['id'] = (int) $product->product_id;

                    if(isset($segments[$count -1]) && isset($segments[$count - 2]) && $segments[$count - 1] == JText::_('COM_JOOMDOWNLOADS_TASK_DOWNLOAD')){
                        $query = $db->getQuery(true)
                            ->select($db->quoteName(array('release_id')))
                            ->from($db->quoteName('#__joomdownloads_release'))
                            ->where($db->quoteName('version') . ' = ' . $db->quote($segments[$count - 2]))
                            ->where($db->quoteName('product_id') . ' = ' . $db->quote($product->product_id));
                        $db->setQuery($query);
                        $release = $db->loadObject();
                       
                        $vars['task'] = 'download';

                        if($release){
                            $vars['release_id'] = $release->release_id;                            
                            return $vars;
                        }
                    }
                    elseif(isset($segments[$count -1]) && $segments[$count - 1] == JText::_('COM_JOOMDOWNLOADS_TASK_RELEASES')){
                    	$vars['task'] = 'releases';
                    	 return $vars;
                    }
                    else{
                        return $vars;
                    }
                }
            }
        }

        JError::raiseError(404, JText::_('COM_JOOMDOWNLOADS_UNEXPECTED_ROUTE'));
    }

    public static function getCategories(){
        static $categories = null;
        if($categories == null){
            $db = JFactory::getDbo();
            $sql = new JDQuery();
            $sql->select('*')
                    ->from('#__joomdownloads_category');                       
            $db->setQuery($sql);
            $categories = $db->loadObjectList('category_id');
        }
        return $categories;
    }
}

/**
 * Joomdownloads router functions
 *
 * These functions are proxys for the new router interface
 * for old SEF extensions.
 *
 * @param   array  &$query  An array of URL arguments
 *
 * @return  array  The URL arguments to use to assemble the subsequent URL.
 *
 * @deprecated  4.0  Use Class based routers instead
 */
function joomdownloadsBuildRoute(&$query)
{
    $router = new JoomdownloadsRouter;

    return $router->build($query);
}

/**
 * Parse the segments of a URL.
 *
 * This function is a proxy for the new router interface
 * for old SEF extensions.
 *
 * @param   array  $segments  The segments of the URL to parse.
 *
 * @return  array  The URL attributes to be used by the application.
 */
function joomdownloadsParseRoute($segments)
{
    $router = new JoomdownloadsRouter;

    return $router->parse($segments);
}
